# coding: utf8
from __future__ import unicode_literals


ADJECTIVES_IRREG = {
    "χειρότερος": ("κακός",),
    "χειρότερη": ("κακός",),
    "χειρότερο": ("κακός",),
    "βέλτιστος": ("καλός",),
    "βέλτιστη": ("καλός",),
    "βέλτιστο": ("καλός",),
    "βέλτιστοι": ("καλός",),
    "βέλτιστων": ("καλός",),
    "ελάχιστος": ("λίγος",),
    "ελάχιστα": ("λίγος",),
    "ελάχιστοι": ("λίγος",),
    "ελάχιστων": ("λίγος",),
    "ελάχιστη": ("λίγος",),
    "ελάχιστο": ("λίγος",),
    "πλείστος": ("πολύς",),
    "πολλή": ("πολύ",),
    "πολύς": ("πολύ",),
    "πολλύ": ("πολύ",),
    "πολλύς": ("πολύ",),

}
